<!doctype html><html lang="en-US"><head><meta charset="utf-8"><title>Amundsen - Data Discovery Portal</title><meta name="description" content=""><meta name="viewport" content="width=device-width,initial-scale=1">{% if env == "development" %} {% include 'fragments/icons-dev.html' %} {% elif env == "staging" %} {% include 'fragments/icons-staging.html' %} {% else %} {% include 'fragments/icons-prod.html' %} {% endif %} {% if false %} {% include 'fragments/google-analytics-loader.html' %} {% endif %}<link href="/static/dist/main.21237cbb02c5af7e95c1.css" rel="stylesheet"></head><body class="{{ env }}"><div id="content"/><script src="/static/dist/vendors.1bf0daca9f744ac21954.js"></script><script src="/static/dist/main.cae920e42693baf8bf49.js"></script></body>{% if false %} {% include 'fragments/google-analytics-post-loader.html' %} {% endif %}</html>