<link rel="apple-touch-icon" sizes="180x180" href="/static/images/favicons/dev/apple-touch-icon.png"><link rel="icon" type="image/png" sizes="32x32" href="/static/images/favicons/dev/favicon-32x32.png"><link rel="icon" type="image/png" sizes="16x16" href="/static/images/favicons/dev/favicon-16x16.png"><link rel="manifest" href="/static/images/favicons/dev/site.webmanifest" crossorigin="use-credentials"><link rel="mask-icon" href="/static/images/favicons/dev/safari-pinned-tab.svg" color="#5bbad5"><link rel="shortcut icon" href="/static/images/favicons/dev/favicon.ico"><meta name="msapplication-TileColor" content="#2d89ef"><meta name="msapplication-config" content="/static/images/favicons/dev/browserconfig.xml"><meta name="theme-color" content="#ffffff">